@@ -6,8 +6,8 @@ class PostgresqlImpl(DefaultImpl):
     __dialect__ = 'postgresql'
     transactional_ddl = True
 
-    def compare_server_default(self, inspector_column, 
-                            metadata_column, 
+    def compare_server_default(self, inspector_column,
+                            metadata_column,
                             rendered_metadata_default):
 
         # don't do defaults for SERIAL columns
@@ -17,6 +17,9 @@ class PostgresqlImpl(DefaultImpl):
 
         conn_col_default = inspector_column['default']
 
+        if None in (conn_col_default, rendered_metadata_default):
+            return conn_col_default != rendered_metadata_default
+
         if metadata_column.type._type_affinity is not sqltypes.String:
             rendered_metadata_default = re.sub(r"^'|'$", "", rendered_metadata_default)
 
